# -*- encoding: utf-8 -*-

import json
from builtins import str as text
from unittest import TestCase

import pytest
from django.contrib.contenttypes.models import ContentType
from mock import MagicMock

from multiseek.logic import (
    AND,
    CONTAINS,
    DIFFERENT,
    EQUAL,
    EQUALITY_OPS_ALL,
    GREATER_OR_EQUAL,
    LESSER_OR_EQUAL,
    MULTISEEK_ORDERING_PREFIX,
    NOT_CONTAINS,
    NOT_STARTS_WITH,
    OR,
    RANGE,
    RANGE_OPS,
    STRING,
    STRING_OPS,
    AbstractNumberQueryObject,
    AutocompleteQueryObject,
    DecimalQueryObject,
    IntegerQueryObject,
    MultiseekRegistry,
    Ordering,
    ParseError,
    QueryObject,
    RangeQueryObject,
    ReportType,
    StringQueryObject,
    UnknownField,
    UnknownOperation,
    create_registry,
    get_registry,
)
from multiseek.models import SearchForm
from multiseek.util import make_field

test_json = json.dumps(
    {
        "form_data": [
            None,
            dict(
                field="foo",
                operator=text(EQUALITY_OPS_ALL[0]),
                value="foo",
                prev_op=None,
            ),
        ]
    }
)

test_buggy_json = json.dumps({"form_data": [None]})

# JSON that's impossible to parse ('text field contains NULL' instead of 'equals NULL')
test_impossible_json = json.dumps(
    {
        "form_data": [
            None,
            dict(
                field="foo",
                operator=text(STRING_OPS[0]),  # "CONTAINS"
                value=None,
                prev_op=None,
            ),
        ]
    }
)


def py3k_test_string(s):
    return s.replace("u'", "'").replace('u"', '"').replace(", u'", ", '")


class TestQueryObject(TestCase):
    def setUp(self):
        class MyQueryObject(QueryObject):
            field_name = "foo"

        self.q = MyQueryObject()

    def test_value_from_web(self):
        self.assertEqual(123, self.q.value_from_web(123))

    def test_query_for(self):
        res = self.q.query_for("foobar", DIFFERENT)
        self.assertEqual(py3k_test_string("(NOT (AND: (u'foo', u'foobar')))"), str(res))

    def test_query_for_raises(self):
        self.assertRaises(
            UnknownOperation, self.q.query_for, "foobar", "unknown operation"
        )


class TestStringQueryObject(TestCase):
    def setUp(self):
        class MyStringQueryObject(StringQueryObject):
            field_name = "foo"

        self.q = MyStringQueryObject()

    def test_query_for(self):
        args = [
            (DIFFERENT, "(NOT (AND: (u'foo', u'foobar')))"),
            (NOT_CONTAINS, "(NOT (AND: (u'foo__icontains', u'foobar')))"),
            (NOT_STARTS_WITH, "(NOT (AND: (u'foo__startswith', u'foobar')))"),
        ]

        for param, result in args:
            res = self.q.real_query("foobar", param)
            self.assertEqual(str(res), py3k_test_string(result))

    def test_query_for_raises(self):
        self.assertRaises(UnknownOperation, self.q.real_query, "lol", "bad operation")


class TestAutocompleteQueryObject(TestCase):
    def test_value_from_web(self):
        q = AutocompleteQueryObject("foo")

        q.model = MagicMock()
        q.model.objects.get.return_value = True
        self.assertEqual(q.value_from_web(None), None)
        self.assertEqual(q.value_from_web("foo"), None)
        self.assertEqual(q.value_from_web(1), True)

    def test_value_to_web(self):
        q = AutocompleteQueryObject("foo")
        q.model = MagicMock()
        q.model.objects.get.return_value = True

        self.assertEqual(q.value_to_web(1), '[1, "True"]')

    def test_value_to_web_None(self):
        q = AutocompleteQueryObject("foo")
        q.model = ContentType
        self.assertEqual(q.value_to_web("null"), '[null, ""]')

    @pytest.mark.django_db
    def test_value_to_web_bug(self):
        q = AutocompleteQueryObject("fo", model=SearchForm)
        self.assertEqual(q.value_to_web(1), '[null, ""]')


class TestAbstractNumberQueryObject(TestCase):
    def test_real_query(self):
        a = AbstractNumberQueryObject("f")
        b = a.real_query(10, DIFFERENT)
        assert str(b) == "(NOT (AND: ('f', 10)))"
        b = a.real_query(10, EQUAL)
        assert str(b) == "(AND: ('f', 10))"


class TestRangeQueryObject(TestCase):
    def test_value_from_web(self):
        r = RangeQueryObject("foo")
        self.assertEqual(r.value_from_web("[1,2]"), [1, 2])
        self.assertEqual(r.value_from_web('["1","2"]'), [1, 2])
        self.assertEqual(r.value_from_web("[1,2,3]"), None)
        self.assertEqual(r.value_from_web('["foo","bar"]'), None)
        self.assertEqual(r.value_from_web("123"), None)

        self.assertRaises(UnknownOperation, r.real_query, [1, 2], "foo")

        res = r.real_query([1, 2], RANGE_OPS[1])

        s1 = "(NOT (AND: (u'foo__gte', 1), (u'foo__lte', 2)))"
        maybe_this = str(res) == py3k_test_string(s1)

        s2 = "(NOT (AND: (u'foo__lte', 2), (u'foo__gte', 1)))"
        maybe_that = str(res) == py3k_test_string(s2)

        self.assert_(maybe_that or maybe_this)


class TestIntegerQueryObject(TestCase):
    def test_value_from_web(self):
        r = IntegerQueryObject("foo")
        self.assertEqual(r.value_from_web("123"), 123)

        self.assertRaises(UnknownOperation, r.real_query, 123, "foo")

        res = r.real_query(123, text(LESSER_OR_EQUAL))
        self.assertEqual(str(res), py3k_test_string("(AND: (u'foo__lte', 123))"))


class _user:
    def return_true(self):
        return True

    is_authenticated = property(return_true)


class FakeRequest:
    def __init__(self):
        self.user = _user()


class TestMultiseekRegistry(TestCase):
    def setUp(self):
        self.registry = MultiseekRegistry()
        self.registry.add_field(StringQueryObject("foo"))
        self.registry.add_field(RangeQueryObject("bar"))
        self.registry.add_field(RangeQueryObject("quux", public=False))
        self.registry.report_types = [
            ReportType("list", "List"),
            ReportType("table", "Table"),
        ]
        self.registry.ordering = [
            Ordering("foo", "Foo"),
            Ordering("bar", "Bar"),
        ]

    def test_add_field_raises(self):
        self.assertRaises(
            AssertionError, self.registry.add_field, StringQueryObject("foo")
        )

    def test_field_by_type(self):
        self.assertEqual(len(self.registry.field_by_type(STRING)), 1)

        self.assertEqual(len(self.registry.field_by_type(RANGE, FakeRequest())), 2)

    def test_extract(self):
        self.assertEqual(self.registry.extract("field_name"), ["foo", "bar"])

        self.assertEqual(
            self.registry.extract("field_name", FakeRequest()), ["foo", "bar", "quux"]
        )

    def test_parse_field(self):
        self.assertRaises(ParseError, self.registry.parse_field, {})

        self.assertRaises(
            UnknownField,
            self.registry.parse_field,
            dict(field="XXX", operator="IS", value="LOL", prev_op=None),
        )

        self.assertRaises(
            UnknownOperation,
            self.registry.parse_field,
            dict(field="foo", operator="XXX", value="FO", prev_op=None),
        )

        res = self.registry.parse_field(
            dict(field="foo", operator=EQUALITY_OPS_ALL[0], value="foo", prev_op=None)
        )

        self.assertEqual(str(res), py3k_test_string("(AND: (u'foo', u'foo'))"))

    def test_get_recursive_list(self):
        input = [
            None,
            [
                None,
                [
                    None,
                    dict(
                        field="foo",
                        operator=text(EQUALITY_OPS_ALL[0]),
                        value="foo",
                        prev_op=None,
                    ),
                ],
            ],
            dict(
                field="foo",
                operator=text(EQUALITY_OPS_ALL[0]),
                value="bar",
                prev_op="BAD OP",
            ),
        ]

        self.registry.get_query(input)
        self.assertEquals(len(self.registry.errors), 1)

        input[2]["prev_op"] = OR

        res = self.registry.get_query(input)
        self.assertEqual(
            str(res), py3k_test_string("(OR: (u'foo', u'foo'), (u'foo', u'bar'))")
        )
        self.assertEquals(len(self.registry.errors), 0)

    def test_get_query_errors(self):
        self.registry.get_query(json.loads(test_impossible_json)["form_data"])
        self.assertEqual(len(self.registry.errors), 1)
        self.assertEqual(self.registry.errors[0][1]["field"], "foo")

    def test_get_query(self):
        gq = self.registry.get_query(json.loads(test_json)["form_data"])
        self.assertEqual(str(gq), py3k_test_string("(AND: (u'foo', u'foo'))"))

    def test_get_query_for_model(self):
        self.registry.model = MagicMock()
        self.registry.get_query_for_model(json.loads(test_json))
        self.registry.get_query_for_model(None)

    def test_get_query_for_model_bug(self):
        self.registry.model = MagicMock()
        self.registry.get_query_for_model(json.loads(test_buggy_json))

    def test_recreate_form(self):
        op = text(EQUALITY_OPS_ALL[0])
        fld_noop = dict(field="foo", operator=op, value=u"foo", prev_op=None)
        fld_and = dict(field="foo", operator=op, value=u"foo", prev_op=AND)
        fld_or = dict(field="foo", operator=op, value=u"foo", prev_op=OR)
        res = self.registry.recreate_form(
            {
                "form_data": [
                    None,
                    fld_noop,
                    fld_or,
                    [AND, fld_noop, fld_or],
                    fld_and,
                    [OR, fld_noop, fld_or],
                ],
                "ordering": {
                    "%s1" % MULTISEEK_ORDERING_PREFIX: "1",
                    "%s1_dir" % MULTISEEK_ORDERING_PREFIX: "1",
                },
                "report_type": "1",
            }
        )

        self.maxDiff = None

        ex = u"""$('#frame-0').multiseekFrame('addField', 'foo', 'equals', 'foo', null);
$('#frame-0').multiseekFrame('addField', 'foo', 'equals', 'foo', 'or');
$('#frame-0').multiseekFrame('addFrame', 'and');
$('#frame-1').multiseekFrame('addField', 'foo', 'equals', 'foo', null);
$('#frame-1').multiseekFrame('addField', 'foo', 'equals', 'foo', 'or');
$('#frame-0').multiseekFrame('addField', 'foo', 'equals', 'foo', 'and');
$('#frame-0').multiseekFrame('addFrame', 'or');
$('#frame-2').multiseekFrame('addField', 'foo', 'equals', 'foo', null);
$('#frame-2').multiseekFrame('addField', 'foo', 'equals', 'foo', 'or');
\t\t$("select[name=order_1] option").eq(1).prop("selected", true);
\t\t$("input[name=order_1_dir]").attr("checked", true);
\t\t$("select[name=_ms_report_type] option").eq(1).prop("selected", true);
\t\tif (window.Foundation) {
\t\t\t$("input[name=order_1_dir]").next().toggleClass("checked", true)
\t\t}\n""" % dict(
            equal=EQUAL
        )

        self.assertEqual(ex, res)

    def test_create_registry(self):
        create_registry(None, StringQueryObject("foo"))

    def test_get_registry(self):
        r = get_registry("test_app.multiseek_registry")
        from test_app.multiseek_registry import registry

        self.assertEqual(r, registry)

    def test_bug_3(self):
        f = self.registry.fields[0]
        v = self.registry.fields[0].ops[0]
        value = "foo"

        field = make_field(f, v, value)
        field_or = make_field(f, v, value, "lol")

        form = {"form_data": [None, field, field, field, field_or]}

        self.assertRaises(ParseError, self.registry.recreate_form, form)


def test_impacts_query_IntegerQueryObject():
    a = IntegerQueryObject("foo")
    assert a.impacts_query(None, EQUAL)
    assert a.impacts_query(None, DIFFERENT)
    assert not a.impacts_query(None, GREATER_OR_EQUAL)


def test_impacts_query_DecimalQueryObject():
    a = DecimalQueryObject("foo")
    assert a.impacts_query(None, EQUAL)
    assert a.impacts_query(None, DIFFERENT)
    assert not a.impacts_query(None, GREATER_OR_EQUAL)


def test_impacts_query_StringQueryObject():
    a = StringQueryObject("foo")
    assert a.impacts_query(None, EQUAL)
    assert a.impacts_query(None, DIFFERENT)
    assert not a.impacts_query(None, CONTAINS)
    assert not a.impacts_query(None, NOT_CONTAINS)



# -*- encoding: utf-8 -*-

import os
import sys
from distutils.command.build import build as _build

from setuptools import Command, setup
from setuptools.command.install_lib import install_lib as _install_lib


class compile_translations(Command):
    description = "compile message catalogs to MO files via django compilemessages"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        curdir = os.getcwd()
        os.chdir(os.path.join(os.path.dirname(__file__), "multiseek"))
        from django.core.management import call_command

        call_command("compilemessages")
        os.chdir(curdir)


class build(_build):
    sub_commands = [("compile_translations", None)] + _build.sub_commands


class install_lib(_install_lib):
    def run(self):
        self.run_command("compile_translations")
        _install_lib.run(self)


# Utility function to read the README file.
# Used for the long_description. It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


class RunTests(Command):
    description = "Run the django test suite from the testproj dir."

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        this_dir = os.getcwd()
        testproj_dir = os.path.join(this_dir, "test_project")
        os.chdir(testproj_dir)
        sys.path.append(testproj_dir)
        from django.core.management import execute_manager

        os.environ["DJANGO_SETTINGS_MODULE"] = "test_project.settings"
        settings_file = os.environ["DJANGO_SETTINGS_MODULE"]
        settings_mod = __import__(settings_file, {}, {}, [""])
        execute_manager(
            settings_mod, argv=[__file__, "test", "multiseek", "--traceback"]
        )
        os.chdir(this_dir)


if "sdist" in sys.argv:
    # clear compiled mo files before building the distribution
    walk = os.walk(os.path.join(os.getcwd(), "multiseek/locale"))
    for dirpath, dirnames, filenames in walk:
        if not filenames:
            continue

        for fn in ["django.mo", "djangojs.mo"]:
            if fn in filenames:
                os.unlink(os.path.join(dirpath, fn))

else:
    # Always try to build messages, fail if django not present
    # (I hate incomplete releases)
    cwd = os.getcwd()
    os.chdir(os.path.join(os.path.dirname(__file__), "multiseek"))
    if "DJANGO_SETTINGS_MODULE" in os.environ:
        del os.environ["DJANGO_SETTINGS_MODULE"]
    os.system("django-admin.py compilemessages")
    os.chdir(cwd)


def reqs(f):
    f = os.path.join(os.path.dirname(__file__), f)
    for elem in open(f).readlines():
        elem = elem.strip()

        if not elem:
            continue

        if elem.find("#egg=") > 0:
            ign, egg = elem.split("#egg=")
            yield egg
            continue

        if elem.startswith("#") or elem.startswith("-r"):
            continue

        yield elem


setup(
    name="django-multiseek",
    version="0.9.45",
    description="Build a form to seek records using multiple parameters",
    author=u"Michał Pasternak",
    author_email="michal.dtz@gmail.com",
    url="http://TODO",
    packages=["multiseek"],
    package_data={
        "multiseek": [
            "locale/*/LC_MESSAGES/*",
            "static/multiseek/*.js",
            "static/multiseek/*.css",
            "templates/multiseek/*.html",
        ]
    },
    include_package_data=True,
    zip_safe=False,
    long_description=read("README.md"),
    license="MIT",
    keywords="django multiseek",
    install_requires=["Django>=1.11"] + list(reqs("requirements.txt")),
    tests_require=["Django>=1.11"] + list(reqs("requirements_dev.txt")),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    cmdclass={
        "build": build,
        "install_lib": install_lib,
        "compile_translations": compile_translations,
        "test": RunTests,
    },
)



# -*- encoding: utf-8 -*-
import json
from builtins import str as text

from django.contrib.auth.models import AnonymousUser, User
from django.test import TestCase
from django.test.client import RequestFactory
from mock import MagicMock
from model_bakery import baker
from test_app import multiseek_registry
from test_app.models import Author

from multiseek.logic import (
    EQUAL,
    EQUALITY_OPS_ALL,
    AutocompleteQueryObject,
    StringQueryObject,
    ValueListQueryObject,
    create_registry,
)
from multiseek.models import SearchForm
from multiseek.views import (
    ERR_FORM_NAME,
    ERR_LOADING_DATA,
    ERR_NO_FORM_DATA,
    ERR_PARSING_DATA,
    MULTISEEK_SESSION_KEY,
    OVERWRITE_PROMPT,
    MultiseekFormPage,
    MultiseekResults,
    MultiseekSaveForm,
    get_registry,
    load_form,
    reset_form,
    user_allowed_to_save_forms,
)


class Session(dict):
    def save(self):
        return None


def setup_anonymous_session(request):
    request.user = AnonymousUser()
    request.session = Session()
    return request


class MockupAutocompleteQueryObject(AutocompleteQueryObject):
    def get_url(self):
        return "/LOL/"


class RegistryMixin:
    def setUp(self):
        self.registry = create_registry(
            None,
            StringQueryObject("foo"),
            StringQueryObject("bar"),
            ValueListQueryObject(field_name="baz", values=["a", "b", "c"]),
            MockupAutocompleteQueryObject(field_name="quux", model=Author),
        )

        self.request = setup_anonymous_session(RequestFactory().get("/"))


class TestViews(RegistryMixin, TestCase):
    def test_new_page(self):
        mfp = MultiseekFormPage(registry=self.registry)
        mfp.request = self.request
        mfp.get_context_data()

    def test_multiseek(self):
        self.request.session[MULTISEEK_SESSION_KEY] = json.dumps(
            {
                "form_data": [
                    None,
                    dict(
                        field="foo",
                        prev_op="or",
                        operator=text(EQUALITY_OPS_ALL[0]),
                        value="foo",
                    ),
                ]
            }
        )

        mfp = MultiseekFormPage(registry=self.registry)
        mfp.request = self.request

        ret = mfp.get_context_data()

        self.assertEqual(ret["js_fields"], '["foo", "bar", "baz", "quux"]')
        self.assertEqual(ret["js_autocompletes"], '{"quux": "/LOL/"}')
        self.assertEqual(ret["js_value_lists"], '{"baz": ["a", "b", "c"]}')
        self.assertEqual(
            ret["js_init"],
            u"$('#frame-0').multiseekFrame('addField', 'foo', 'equals', 'foo', 'or');\n",
        )

    def test_reset_form(self):
        self.request.session[MULTISEEK_SESSION_KEY] = "123"
        reset_form(self.request)

        self.assertEqual(self.request.session, {})

        # No error on subsequent calls
        reset_form(self.request)

    def test_get_registry(self):
        self.assertEqual(get_registry({}), {})

        self.assertEqual(
            get_registry("test_app.multiseek_registry"), multiseek_registry.registry
        )

    def test_user_allowed_to_save_forms(self):
        class MockUser:
            is_staff = True

        self.assertEqual(user_allowed_to_save_forms(MockUser), MockUser.is_staff)
        MockUser.is_staff = False
        self.assertEqual(user_allowed_to_save_forms(MockUser), MockUser.is_staff)

        class MockUserWithout:
            pass

        self.assertEqual(user_allowed_to_save_forms(MockUserWithout), None)


class TestMultiseekSaveForm(RegistryMixin, TestCase):
    def setUp(self):
        RegistryMixin.setUp(self)
        self.msp = MultiseekSaveForm(registry=self.registry)
        self.msp.request = self.request

    def test_save_form_anon_user(self):
        res = self.msp.post(self.request)
        self.assertEqual(res.status_code, 403)

        self.assertEqual(self.msp.post, self.msp.get)

    def test_get_context_data(self):
        self.request.POST = {}

        self.request.POST["json"] = None
        self.assertEqual(
            self.msp.get_context_data(), dict(result=text(ERR_NO_FORM_DATA))
        )

        self.request.POST["json"] = "wcale, nie, json"
        self.assertEqual(
            self.msp.get_context_data(), dict(result=text(ERR_PARSING_DATA))
        )

        self.request.POST["json"] = '[{"field": "foo", "bad": "field"}]'
        self.assertEqual(
            self.msp.get_context_data(), dict(result=text(ERR_LOADING_DATA))
        )

        self.request.POST["json"] = (
            '{"form_data": [{"field": "foo", "operation": "'
            + text(EQUAL)
            + '", "value": "foo"}]}'
        )
        self.request.POST["name"] = ""
        self.assertEqual(self.msp.get_context_data(), dict(result=text(ERR_FORM_NAME)))

        baker.make(SearchForm, name="foo")
        self.request.POST["name"] = "foo"
        self.assertEqual(self.msp.get_context_data(), dict(result=OVERWRITE_PROMPT))

        self.request.POST["overwrite"] = "true"
        self.request.user = baker.make(User)
        self.assertEqual(
            self.msp.get_context_data()["result"], "saved"
        )  # dict(result=SAVED, pk=1))

        self.assertEqual(SearchForm.objects.all().count(), 1)
        self.assertEqual(SearchForm.objects.all()[0].public, False)

        self.request.POST["public"] = "true"
        self.msp.get_context_data()
        self.assertEqual(SearchForm.objects.all()[0].public, True)


class TestMultiseekLoadForm(TestCase):
    def setUp(self):
        self.user = User.objects.create_user("test", "test@foo.pl", "test")

        self.anon_req = setup_anonymous_session(RequestFactory().get("/"))

        self.normal_req = RequestFactory().get("/")
        self.normal_req.user = self.user
        self.normal_req.session = Session()

    def test_load_form_unexistent(self):
        res = load_form(self.anon_req, 1)
        self.assertEqual(res.status_code, 404)

    def test_load_form_existent_public_ok(self):
        sf = SearchForm.objects.create(
            name="foo", owner=self.user, public=True, data="some data"
        )
        res = load_form(self.anon_req, sf.pk)
        self.assertEqual(res.status_code, 302)
        self.assertEqual(self.anon_req.session[MULTISEEK_SESSION_KEY], sf.data)

    def test_load_form_forbidden(self):
        sf = SearchForm.objects.create(
            name="foo", owner=self.user, public=False, data="some data"
        )
        res = load_form(self.anon_req, sf.pk)
        self.assertEqual(res.status_code, 403)

    def test_load_form_non_public_logged_in_user(self):
        sf = SearchForm.objects.create(
            name="foo", owner=self.user, public=False, data="some data"
        )
        res = load_form(self.normal_req, sf.pk)
        self.assertEqual(res.status_code, 302)


class TestMultiseekResults(RegistryMixin, TestCase):
    def setUp(self):
        RegistryMixin.setUp(self)
        self.registry.model = MagicMock()
        self.mr = MultiseekResults(registry=self.registry)
        self.mr.request = self.request
        self.request.session[MULTISEEK_SESSION_KEY] = json.dumps(
            {
                "form_data": [
                    None,
                    {
                        "field": text(self.registry.fields[0].label),
                        "operator": text(self.registry.fields[0].ops[0]),
                        "value": u"foobar",
                    },
                ]
            }
        )

    def test_post(self):
        res = self.mr.post(self.request)
        self.assertEqual(res.status_code, 200)

    def test_get_queryset(self):
        self.mr.get_queryset()

    def test_describe_multiseek_data(self):
        self.mr.post(self.request)
        res = self.mr.describe_multiseek_data()
        self.assertEqual(res, 'foo contains "foobar"')
